Option Explicit

'#Uses "Strings.bas"
'#Uses "CountryCode.cls"


' @User: This contant must contain the full path and file name of ExifTool on your machine
'
'        You can download ExifTool via http://www.sno.phy.queensu.ca/~phil/exiftool/
'
Private Const ET_PATH = "D:\Bilder_Videos\exiftool\exiftool.exe"
Private Const outfile = Environ("TEMP") & "\output.tmp"
Private Const outfile1 = Environ("TEMP") & "\output1.tmp"
Private Const NeededMaker = "Panasonic"
Private Const NeededModel = "DMC-TZ10"

Const Version = "1.02fad"

' ------------------------------------------------
' - Prft ob die Datei existiert
' -
' - Filename = Pfad und Dateinamen der geprft
' -            werden soll
' ------------------------------------------------
Private Function FileExists(Filename As String) As Boolean

    If Filename <> "" Then
        FileExists = (Dir$(Filename, vbHidden ) <> "")
    Else
        FileExists = False
    End If

End Function

Private Function IsFileReady(ByVal Filename As String) As Boolean

    Dim result As Boolean
    result = False

	While Not FileExists(Filename)
	Wend

    While  FileLen(Filename) = 0
    Wend

    result = True

    IsFileReady = result

End Function


' ----------------------------------------------------
' - Auslesen des Maker Tag und des Models aus den
' - Exif Informationen mit IMatch Mitteln.
' -
' - Die Variable 'make' sollte wenn alles OK ist
' - 'Panasonic' und die Variable 'model' den Wert
' - 'DMC-TZ10' enthalten. Ist dies der Fall so liefert
' - die Funktion True zurck.
'
' - db    = die zu verwendende IMatch Datenbank
' - Image = das zu verwendende Bild
' -
' - gendert am 02.04.2010
' ----------------------------------------------------
Private Function ReadMaker(ByVal db As Database, ByVal Image As Image) As Boolean

    Dim make As String
    Dim model As String

    make = db.ParseVariables("{Image.EXIF.Make}",Image)
	model = db.ParseVariables("{Image.EXIF.Model}",Image)

	' Debug.Print make
	' Debug.Print model
    ' Debug.Print ((make=NeededMaker) And (model=NeededModel))

	ReadMaker = ((make=NeededMaker) And (model=NeededModel))

End Function

' ------------------------------------------------------
' - Auslesen der Location Information aus dem MakerNote
' - Tag.
' -
' - Input:
' -   Filename = Datei aus der gelesen werden soll
' -
' - Output:
'     Country = das Land
'     State   = das Bundesland
'     City    = die Stadt
'     Landmark= die nchst gelegene PointOfInterest
' ------------------------------------------------------
Private Sub ReadLocationInfo(ByVal Filename As String, _
                             ByRef Country As String, _
	                         ByRef State As String, _
							 ByRef City As String, _
							 ByRef Landmark As String)
	Dim result(3) As String
    Dim cline As String
	Dim i As Integer

	' The ExifTool syntay may look a bit strange, but it is really powerful.
	Const ET_CMDLINE = "#ET_PATH" + " -s -s -s -Country -State -City -Landmark " + """#FILENAME""" & " > #OUTFILE"

	cline =	ET_CMDLINE
    cline = Replace(cline,"#ET_PATH",ET_PATH) ' Pfad zu exiftool
    cline = Replace(cline,"#FILENAME",Filename)
    cline = Replace(cline,"#OUTFILE", outfile1)

    ' exiftool aufrufen und Ausgabe in Datei umleiten
    Shell (Environ$("COMSPEC") & " /c " & cline, vbHide )

    If IsFileReady(outfile1) Then
    	Open outfile1 For Input As #2

    	While Not EOF(2)
    		For i=0 To 3
				Line Input #2, result(i)
    		Next
    	Wend

		Close #2
    End If

    Shell (Environ$("COMSPEC") & " /c del " & outfile1)

    Country=result(0)
    State=result(1)
    City=result(2)
    Landmark=result(3)

End Sub


Sub Main
	Dim db As Database
	Set db = Application.ActiveDatabase

	' ---------------------------------------
	' ist eine Datenbank geffnet ?
	' wenn nicht Script beenden.
	' ---------------------------------------
	If db Is Nothing Then
		MsgBox "Please open a database first!
		Exit Sub
	End If

	Dim iset As Images
	Set iset = db.ActiveSelection
	Dim i As Image
	Dim count As Long
  	Dim Country As String
  	Dim State As String
  	Dim City As String
  	Dim Landmark As String

	Application.WaitDialogOpen "Processing Images",0,iset.Count,True
	count = 0

	Dim CountryAndCode As New CountryCode

  	' We need an IPTCStream object to access the files
	Dim iptc As IPTCStream
	Set iptc = New IPTCStream

	' This variable holds results from our IPTC stream
	Dim res As IMIPTCResult

	db.BeginTransaction

  	For Each i In iset
		Application.WaitDialogSetText "Processing " + i.FileName
		' Allow the user to abort the operation
		If Application.WaitDialogIsCanceled Then
			Exit For
		End If

		If ReadMaker(db, i) Then
		 ' Debug.Print i.FileName
		  ReadLocationInfo(i.FileName,Country,State,City,Landmark)

		  ' Debug.Print Country ' {Image.IPTC.Country/Primary Location Name}
  		  ' Debug.Print State
  		  ' Debug.Print City
  		  ' Debug.Print Landmark
  		  ' Debug.Print "------------------"
  		End If

  		' Read the IPTC information from the file. This way we
		' know its accessible and has a supported format
		res = iptc.ReadFile(i.FileName)

		' If we're successful (may also mean that there currently is no IPTC info)
		If res = imiptcSuccess Or res = imiptcNoIPTCInfo Then

			iptc.Fields.Item("Country/Primary Location Name") = PCase(Country)
			iptc.Fields.Item("Province/State") = PCase(State)
			iptc.Fields.Item("City") = PCase(City)
			iptc.Fields.Item("Sublocation") = PCase(Landmark)

			If CountryAndCode.IsCountryCode(Country) Then
				iptc.Fields.Item("Country/Primary Location Code") = CountryAndCode.Code
			End If

			res = imiptcSuccess
			res = iptc.WriteFile(i.FileName)
			db.RefreshImage i

			' Check the result
			If res <> imiptcSuccess Then
				MsgBox "Error writing the IPTC Information to the file " & i.FileName
			End If
		Else
			Debug.Print "Skipping file " & i.FileName
		End If

		count = count + 1
		If Application.WaitDialogIsCanceled Then
			Exit For
		End If

		Application.WaitDialogSetPercentage count
  	Next

  	db.EndTransaction True

	Application.WaitDialogSetPercentage count

	db.Redraw = True

End Sub
